#!/bin/sh
# Copyright (c) 2000-2015 Synology Inc. All rights reserved.

. "`/usr/bin/dirname $0`"/dnsfilter-util.sh

#
# Redirect port numbers.
#
# If a user tries to access a blocked domain name, the user will
# be redirected to the specified port.
#
# DNSFILTER_REDIR_PORT_HTTP: redirect port number of HTTP requests
# DNSFILTER_REDIR_PORT_HTTPS: redirect port number of HTTPS requests
#
DNSFILTER_REDIR_PORT_HTTP=5010
DNSFILTER_REDIR_PORT_HTTPS=5011
SECURITY_REDIR_PORT_HTTP=5012
SECURITY_REDIR_PORT_HTTPS=5013
GOOGLE_SAFE_BROWSING_REDIR_PORT_HTTP=5014
GOOGLE_SAFE_BROWSING_REDIR_PORT_HTTPS=5015

#
# The redirect IP address replies to the blocked DNS requests.
#
DNSFILTER_BLOCK_REDIR_IP="10.254.1.2"
DNSFILTER_BLOCK_REJECT_IP="10.254.1.3"
SECURITY_BLOCK_REDIR_IP="10.254.2.2"
SECURITY_BLOCK_REJECT_IP="10.254.2.3"
GOOGLE_SAFE_BROWSING_BLOCK_REDIR_IP="10.254.5.2"
GOOGLE_SAFE_BROWSING_BLOCK_REJECT_IP="10.254.5.3"

#
# The redirect IPv6 address replies to the blocked DNS requests.
#
DNSFILTER_BLOCK_REDIR_IPV6="fc00:eeee:1::2"
DNSFILTER_BLOCK_REJECT_IPV6="fc00:eeee:1::3"
SECURITY_BLOCK_REDIR_IPV6="fc00:eeee:2::2"
SECURITY_BLOCK_REJECT_IPV6="fc00:eeee:2::3"
GOOGLE_SAFE_BROWSING_BLOCK_REDIR_IPV6="fc00:eeee:5::2"
GOOGLE_SAFE_BROWSING_BLOCK_REJECT_IPV6="fc00:eeee:5::3"

#
# The iptables chain name used by the URL blocker.
#
CHAIN_NAME_NAT="URL_BLOCKER"
CHAIN_NAME_FILTER="URL_BLOCKER_FILTER"
CHAIN_NAME_FILTER_FORWARD="URL_BLOCKER_FORWARD_FILTER"

#
# The main URL blocker's configuration file.
#
BIN_IPTABLES_V4="/sbin/iptables"
BIN_IPTABLES_V6="/sbin/ip6tables"

#
# Script name, used to display error message
#
SCRIPT_NAME=`/usr/bin/basename "$0"`

RETVAL=0

#
# Support ipv6, whether this platform support ipv6 or not
#
SUPPORT_IPV6=`get_support_ipv6`

#
# Add routing rules to redirect blocked HTTP and HTTPS requests
# to the local HTTP/HTTPS server.
#
# @param server_ip [IN]
#		The IP address of the local HTTP/HTTPS server.
# @param netmask [IN]
#		The netmask of the IP address.
# @param redir_ip [IN]
#		The redirect IP for the HTTP/HTTPS requests.
# @param reject_ip [IN]
#		The reject IP for the cumstomized HTTPS requests.
# @param redir_port_http [IN]
#		The redirect port for HTTP requests.
# @param redir_port_https [IN]
#		The redirect port for HTTPS requests.
#
add_redirect_rules_v4 ()
{
	local server_ip="$1"
	local netmask="$2"
	local redir_ip="$3"
	local reject_ip="$4"
	local redir_port_http="$5"
	local redir_port_https="$6"

	if [ $# -lt 6 ]; then
		/usr/bin/logger -p err "${SCRIPT_NAME}: add_redirect_rules_v4() failed - bad parameters"
		return 1
	fi

	if [ "x" != "x${server_ip}" ]; then
		# redirect HTTP requests
		${BIN_IPTABLES_V4} -t nat -A ${CHAIN_NAME_NAT} -p tcp \
				-s ${server_ip}/${netmask} \
				-d ${redir_ip}/32 --dport 80 -j DNAT \
				--to-destination ${server_ip}:${redir_port_http}
		# redirect HTTPS requests
		${BIN_IPTABLES_V4} -t nat -A ${CHAIN_NAME_NAT} -p tcp \
				-s ${server_ip}/${netmask} \
				-d ${redir_ip}/32 --dport 443 -j DNAT \
				--to-destination ${server_ip}:${redir_port_https}

		# redirect HTTP requests for not default redirect IP
		${BIN_IPTABLES_V4} -t nat -A ${CHAIN_NAME_NAT} -p tcp \
				-s ${server_ip}/${netmask} \
				-d ${redir_ip}/24 --dport 80 -j DNAT \
				--to-destination ${server_ip}:${redir_port_http}
		# reject HTTPS requests for customized reject IP
		${BIN_IPTABLES_V4} -t filter -A ${CHAIN_NAME_FILTER_FORWARD} -p tcp \
				-s ${server_ip}/${netmask} \
				-d ${reject_ip} --dport 443 -j REJECT --reject-with tcp-reset

		# avoid bridge mode passing by SYNO_FORWARD_ACCEPT
		${BIN_IPTABLES_V4} -t filter -I SYNO_FORWARD_ACCEPT -j ${CHAIN_NAME_FILTER_FORWARD}
	fi

	return 0
}

#
# Add ipv6 redirect iptable rules to redirect blocked HTTP and HTTPS requests
# to the local HTTP/HTTPS server.
#
# @param ifname [IN]
#		The network interface of the HTTP/HTTPS requests.
# @param server_ip [IN]
#		The IP address of the local HTTP/HTTPS server.
# @param netmask [IN]
#		The netmask of the IP address.
# @param redir_ip [IN]
#		The redirect IP for the HTTP/HTTPS requests.
# @param reject_ip [IN]
#		The reject IP for the cumstomized HTTPS requests.
# @param redir_port_http [IN]
#		The redirect port for HTTP requests.
# @param redir_port_https [IN]
#		The redirect port for HTTPS requests.
#
add_redirect_rules_v6 ()
{
	local ifname="$1"
	local server_ipv6="$2"
	local server_prefix="$3"
	local redir_ipv6="$4"
	local reject_ipv6="$5"
	local redir_port_http="$6"
	local redir_port_https="$7"

	if [ $# -lt 7 ] || \
		[ -z ${server_ipv6} -o -z ${server_prefix} ] || \
		[ -z ${redir_ipv6} -o -z ${reject_ipv6} ] || \
		[ -z ${redir_port_http} -o -z ${redir_port_https} ]; then
		/usr/bin/logger -p warn "${SCRIPT_NAME}: add_redirect_rules_v6() failed - bad parameters"
		return 1
	fi

	if [ "xyes" != "x${SUPPORT_IPV6}" ]; then
		return 0
	fi

	# redirect HTTP requests
	${BIN_IPTABLES_V6} -t nat -A ${CHAIN_NAME_NAT} \
		-i ${ifname} \
		-d ${redir_ipv6} \
		-p tcp --dport 80 \
		-j DNAT --to-destination [${server_ipv6}]:${redir_port_http}

	# redirect HTTPS requests
	${BIN_IPTABLES_V6} -t nat -A ${CHAIN_NAME_NAT} \
		-i ${ifname} \
		-d ${redir_ipv6} \
		-p tcp --dport 443 \
		-j DNAT --to-destination [${server_ipv6}]:${redir_port_https}

	# redirect HTTP requests for not default redirect IP
	${BIN_IPTABLES_V6} -t nat -A ${CHAIN_NAME_NAT} \
		-i ${ifname} \
		-d ${redir_ipv6}/${server_prefix} \
		-p tcp --dport 80 \
		-j DNAT --to-destination [${server_ipv6}]:${redir_port_http}

	# reject HTTPS requests for customized reject IP
	${BIN_IPTABLES_V6} -t filter -A ${CHAIN_NAME_FILTER_FORWARD} \
		-i ${ifname} \
		-d ${reject_ipv6} \
		-p tcp --dport 443 \
		-j REJECT --reject-with tcp-reset

	# avoid bridge mode passing by SYNO_FORWARD_ACCEPT
	${BIN_IPTABLES_V6} -t filter -I SYNO_FORWARD_ACCEPT -j ${CHAIN_NAME_FILTER_FORWARD}

	return 0
}

#
# Add accept port to redirect blocked HTTP/HTTPS requests
# to the local HTTP/HTTPS server.
#
# @param ifname [IN]
#		The network interface of the HTTP/HTTPS requests.
# @param protocol [IN]
#		TCP or UDP
# @param accept_port [IN]
#		The accept port for redirect / reject ports
#
add_accept_port_rules()
{
	local ifname="$1"
	local protocol="$2"
	local accept_port="$3"

	if [ $# -lt 3 ] || [ "x" = "x${ifname}" ] || \
		[ "xtcp" != "x${protocol}" -a "xudp" != "x${protocol}" ]; then
		/usr/bin/logger -p err "${SCRIPT_NAME}: add_accept_port_rules() failed - bad parameters"
		return 1
	fi

	${BIN_IPTABLES_V4} -t filter -A ${CHAIN_NAME_FILTER} \
		-i ${ifname} -p ${protocol} --dport ${accept_port} -j ACCEPT

	# IPv6 accept redirect requests sent from ${ifname}
	if [ "xyes" = "x${SUPPORT_IPV6}" ]; then
		${BIN_IPTABLES_V6} -t filter -A ${CHAIN_NAME_FILTER} \
			-i ${ifname} -p ${protocol} --dport ${accept_port} -j ACCEPT
	fi

	return 0
}

#
# Add drop port to redirect blocked HTTP/HTTPS requests
# to the local HTTP/HTTPS server.
#
# @param drop_port [IN]
#		The drop port for redirect / reject ports
#
add_drop_port_rules()
{
	local drop_port="$1"

	${BIN_IPTABLES_V4} -t filter -A ${CHAIN_NAME_FILTER} -p tcp --dport ${drop_port} -j DROP
	${BIN_IPTABLES_V4} -t filter -A ${CHAIN_NAME_FILTER} -p udp --dport ${drop_port} -j DROP

	# IPv6 drop redirect requests sent
	if [ "xyes" = "x${SUPPORT_IPV6}" ]; then
		${BIN_IPTABLES_V6} -t filter -A ${CHAIN_NAME_FILTER} -p tcp --dport ${drop_port} -j DROP
		${BIN_IPTABLES_V6} -t filter -A ${CHAIN_NAME_FILTER} -p udp --dport ${drop_port} -j DROP
	fi

	return 0
}

#
# Apply iptable rules of local bridge interface
#
# @param ifname [IN] The local bridge interface name
#
# @return 1 if error, 0 if success
#
apply_bridge_iptable ()
{
	local ifname="$1"
	local ipv4=""
	local ipv4_mask=""
	local ipv6=""
	local ipv6_prefix=""
	local ipv6_config=""

	if [ "x" = "x${ifname}" ]; then
		return 1
	fi

	# apply ipv4 rules
	ipv4=`/sbin/ifconfig ${ifname} | grep -Eo 'inet (addr:)?([0-9]*\.){3}[0-9]*' | grep -Eo '([0-9]*\.){3}[0-9]*'`
	ipv4_mask=`/sbin/ifconfig ${ifname} | grep -Eo 'Mask:([0-9]*\.){3}[0-9]*' | tail -c +6`
	if [ "x" != "x${ipv4}" -a "x" != "x${ipv4_mask}" ]; then
		add_redirect_rules_v4 ${ipv4} ${ipv4_mask} ${DNSFILTER_BLOCK_REDIR_IP} ${DNSFILTER_BLOCK_REJECT_IP} ${DNSFILTER_REDIR_PORT_HTTP} ${DNSFILTER_REDIR_PORT_HTTPS}
		add_redirect_rules_v4 ${ipv4} ${ipv4_mask} ${SECURITY_BLOCK_REDIR_IP} ${SECURITY_BLOCK_REJECT_IP} ${SECURITY_REDIR_PORT_HTTP} ${SECURITY_REDIR_PORT_HTTPS}
		add_redirect_rules_v4 ${ipv4} ${ipv4_mask} ${GOOGLE_SAFE_BROWSING_BLOCK_REDIR_IP} ${GOOGLE_SAFE_BROWSING_BLOCK_REJECT_IP} ${GOOGLE_SAFE_BROWSING_REDIR_PORT_HTTP} ${GOOGLE_SAFE_BROWSING_REDIR_PORT_HTTPS}
	fi

	# apply ipv6 rules
	if [ "xyes" = "x${SUPPORT_IPV6}" ]; then
		ipv6_config=`/sbin/ip -6 addr show dev ${ifname} scope global | grep -Eo 'inet6 ([0-9a-f]*:{1,2})+([0-9a-f]*)*/([0-9])+' | /usr/bin/awk '{print $2}'`
		ipv6=`echo ${ipv6_config} | /usr/bin/cut -d'/' -f1`
		ipv6_prefix=`echo ${ipv6_config} | /usr/bin/cut -d'/' -f2`

		if [ "x" != "x${ipv6}" -a "x" != "x${ipv6_prefix}" ]; then
			add_redirect_rules_v6 ${ifname} ${ipv6} ${ipv6_prefix} ${DNSFILTER_BLOCK_REDIR_IPV6} ${DNSFILTER_BLOCK_REJECT_IPV6} ${DNSFILTER_REDIR_PORT_HTTP} ${DNSFILTER_REDIR_PORT_HTTPS}
			add_redirect_rules_v6 ${ifname} ${ipv6} ${ipv6_prefix} ${SECURITY_BLOCK_REDIR_IPV6} ${SECURITY_BLOCK_REJECT_IPV6} ${SECURITY_REDIR_PORT_HTTP} ${SECURITY_REDIR_PORT_HTTPS}
			add_redirect_rules_v6 ${ifname} ${ipv6} ${ipv6_prefix} ${GOOGLE_SAFE_BROWSING_BLOCK_REDIR_IPV6} ${GOOGLE_SAFE_BROWSING_BLOCK_REJECT_IPV6} ${GOOGLE_SAFE_BROWSING_REDIR_PORT_HTTP} ${GOOGLE_SAFE_BROWSING_REDIR_PORT_HTTPS}
		fi
	fi

	add_accept_port_rules ${ifname} tcp ${DNSFILTER_REDIR_PORT_HTTP}
	add_accept_port_rules ${ifname} tcp ${DNSFILTER_REDIR_PORT_HTTPS}
	add_accept_port_rules ${ifname} tcp ${SECURITY_REDIR_PORT_HTTP}
	add_accept_port_rules ${ifname} tcp ${SECURITY_REDIR_PORT_HTTPS}
	add_accept_port_rules ${ifname} tcp ${GOOGLE_SAFE_BROWSING_REDIR_PORT_HTTP}
	add_accept_port_rules ${ifname} tcp ${GOOGLE_SAFE_BROWSING_REDIR_PORT_HTTPS}

	return 0
}

#
# Apply iptable to guest bridge interface
#
# @param ifname [IN] The guest bridge interface name
#
# @return 1 if error, 0 if success
#
apply_guest_bridge_iptable ()
{
	local ifname="$1"
	local ipv4=""
	local ipv4_mask=""
	local ipv6=""
	local ipv6_prefix=""
	local ipv6_config=""

	if [ "x" = "x${ifname}" ]; then
		return 1
	fi

	# apply ipv4 redirect rules
	ipv4=`/sbin/ifconfig ${ifname} | grep -Eo 'inet (addr:)?([0-9]*\.){3}[0-9]*' | grep -Eo '([0-9]*\.){3}[0-9]*'`
	ipv4_mask=`/bin/get_key_value /etc/sysconfig/network-scripts/ifcfg-${ifname} NETMASK`
	if [ "x" != "x${ipv4}" -a "x" != "x${ipv4_mask}" ]; then
		add_redirect_rules_v4 ${ipv4} ${ipv4_mask} ${DNSFILTER_BLOCK_REDIR_IP} ${DNSFILTER_BLOCK_REJECT_IP} ${DNSFILTER_REDIR_PORT_HTTP} ${DNSFILTER_REDIR_PORT_HTTPS}
		add_redirect_rules_v4 ${ipv4} ${ipv4_mask} ${SECURITY_BLOCK_REDIR_IP} ${SECURITY_BLOCK_REJECT_IP} ${SECURITY_REDIR_PORT_HTTP} ${SECURITY_REDIR_PORT_HTTPS}
		add_redirect_rules_v4 ${ipv4} ${ipv4_mask} ${GOOGLE_SAFE_BROWSING_BLOCK_REDIR_IP} ${GOOGLE_SAFE_BROWSING_BLOCK_REJECT_IP} ${GOOGLE_SAFE_BROWSING_REDIR_PORT_HTTP} ${GOOGLE_SAFE_BROWSING_REDIR_PORT_HTTPS}
	fi

	# apply ipv6 redirect rules
	if [ "xyes" = "x${SUPPORT_IPV6}" ]; then
		ipv6_config=`/sbin/ip -6 addr show dev ${ifname} scope global | grep -Eo 'inet6 ([0-9a-f]*:{1,2})+([0-9a-f]*)*/([0-9])+' | /usr/bin/awk '{print $2}'`
		ipv6=`echo ${ipv6_config} | /usr/bin/cut -d'/' -f1`
		ipv6_prefix=`echo ${ipv6_config} | /usr/bin/cut -d'/' -f2`
		if [ "x" != "x${ipv6}" -a "x" != "x${ipv6_prefix}" ]; then
			add_redirect_rules_v6 ${ifname} ${ipv6} ${ipv6_prefix} ${DNSFILTER_BLOCK_REDIR_IPV6} ${DNSFILTER_BLOCK_REJECT_IPV6} ${DNSFILTER_REDIR_PORT_HTTP} ${DNSFILTER_REDIR_PORT_HTTPS}
			add_redirect_rules_v6 ${ifname} ${ipv6} ${ipv6_prefix} ${SECURITY_BLOCK_REDIR_IPV6} ${SECURITY_BLOCK_REJECT_IPV6} ${SECURITY_REDIR_PORT_HTTP} ${SECURITY_REDIR_PORT_HTTPS}
			add_redirect_rules_v6 ${ifname} ${ipv6} ${ipv6_prefix} ${GOOGLE_SAFE_BROWSING_BLOCK_REDIR_IPV6} ${GOOGLE_SAFE_BROWSING_BLOCK_REJECT_IPV6} ${GOOGLE_SAFE_BROWSING_REDIR_PORT_HTTP} ${GOOGLE_SAFE_BROWSING_REDIR_PORT_HTTPS}
		fi
	fi

	add_accept_port_rules ${ifname} tcp ${DNSFILTER_REDIR_PORT_HTTP}
	add_accept_port_rules ${ifname} tcp ${DNSFILTER_REDIR_PORT_HTTPS}
	add_accept_port_rules ${ifname} tcp ${SECURITY_REDIR_PORT_HTTP}
	add_accept_port_rules ${ifname} tcp ${SECURITY_REDIR_PORT_HTTPS}
	add_accept_port_rules ${ifname} tcp ${GOOGLE_SAFE_BROWSING_REDIR_PORT_HTTP}
	add_accept_port_rules ${ifname} tcp ${GOOGLE_SAFE_BROWSING_REDIR_PORT_HTTPS}

	return 0
}

insert_kernel_module ()
{
	if [ "xyes" = "x${SUPPORT_IPV6}" ]; then
		/sbin/insmod /usr/lib/modules/ip6t_REJECT.ko
	fi
}

remove_kernel_module ()
{
	if [ "xyes" = "x${SUPPORT_IPV6}" ]; then
		/sbin/rmmod ip6t_REJECT
	fi
}

#
# Start the URL blocker (or DNS filter) service
#
# There are at most three URL blocker processes running at the same time.
# 1. URL blocker for "basic" profile
# 2. URL blocker for "protected" profile
# 3. URL blocker for "custom" profile
#
# @see url_blocker_stop
#
url_blocker_start ()
{
	local lbr_name gbr_name br_name

	# try getting interfaces' name multiple times in case interface is not ready
	local try_count=0
	while ! ( ([ "${lbr_name}" ] || [ "${br_name}" ]) && [ "${gbr_name}" ] ); do
		[ ${try_count} -gt 4 ] && break
		sleep 1
		lbr_name="`ifconfig | grep -E ^lbr | head -n 1 | cut -d' ' -f1`"
		gbr_name="`ifconfig | grep -E ^gbr | head -n 1 | cut -d' ' -f1`"
		br_name="`ifconfig | grep -E ^br | head -n 1 | cut -d' ' -f1`"
		try_count=$((try_count+1))
	done
	if [ -z "${lbr_name}" ] && [ -z "${br_name}" ]; then
		/usr/bin/logger -p warn "${SCRIPT_NAME}: no (local) bridge interface"
	fi

	# stop the existing URL blocker before starting
	url_blocker_stop

	# install required kernel module
	insert_kernel_module

	# apply iptable for local bridge
	for if_name in "${lbr_name}" "${br_name}"; do
		if [ "x" != "x${if_name}" ]; then
			apply_bridge_iptable ${if_name}
		fi
	done

	# apply iptable for guest bridge
	if [ "x" != "x${gbr_name}" ]; then
		apply_guest_bridge_iptable ${gbr_name}
	fi

	add_drop_port_rules ${DNSFILTER_REDIR_PORT_HTTP}
	add_drop_port_rules ${DNSFILTER_REDIR_PORT_HTTPS}
	add_drop_port_rules ${SECURITY_REDIR_PORT_HTTP}
	add_drop_port_rules ${SECURITY_REDIR_PORT_HTTPS}
	add_drop_port_rules ${GOOGLE_SAFE_BROWSING_REDIR_PORT_HTTP}
	add_drop_port_rules ${GOOGLE_SAFE_BROWSING_REDIR_PORT_HTTPS}
}

#
# Stop the URL blocker (or DNS filter) service.
#
# @see url_blocker_start
#
url_blocker_stop ()
{
	# avoid bridge mode passing by SYNO_FORWARD_ACCEPT
	${BIN_IPTABLES_V4} -t filter -D SYNO_FORWARD_ACCEPT -j ${CHAIN_NAME_FILTER_FORWARD} >/dev/null 2>&1

	${BIN_IPTABLES_V4} -t nat -F ${CHAIN_NAME_NAT} >/dev/null 2>&1
	${BIN_IPTABLES_V4} -t filter -F ${CHAIN_NAME_FILTER} >/dev/null 2>&1
	${BIN_IPTABLES_V4} -t filter -F ${CHAIN_NAME_FILTER_FORWARD} >/dev/null 2>&1

	if [ "xyes" = "x${SUPPORT_IPV6}" ]; then
		# avoid bridge mode passing by SYNO_FORWARD_ACCEPT
		${BIN_IPTABLES_V6} -t filter -D SYNO_FORWARD_ACCEPT -j ${CHAIN_NAME_FILTER_FORWARD} >/dev/null 2>&1

		${BIN_IPTABLES_V6} -t nat -F ${CHAIN_NAME_NAT} >/dev/null 2>&1
		${BIN_IPTABLES_V6} -t filter -F ${CHAIN_NAME_FILTER} >/dev/null 2>&1
		${BIN_IPTABLES_V6} -t filter -F ${CHAIN_NAME_FILTER_FORWARD} >/dev/null 2>&1
	fi

	remove_kernel_module
}

#
# The main function of the URL blocker.
#
# The function use file lock to make sure that the URL blocker
# will not be executed simultaneously.
#
# @param bin_name [IN]
#		The file name of this script. Used to display error message only.
# @param action [IN]
#		Specify what action to do, e.g. start, stop, restart.
#
action_main ()
{
	local action="$1"
	local szf_script_lock="/tmp/url-blocker.sh.lock"

	if [ $# -lt 1 ]; then
		/usr/bin/logger -p err "${SCRIPT_NAME}: main() failed - bad parameters"
		exit 1
	fi

	(
		local retry_max=100
		local retry_count=1

		while [ ${retry_count} -le ${retry_max} ]; do
			if flock -n -x 8; then
				# use file lock to make sure that this script will
				# not be executed simultaneously

				case "${action}" in
				start)
					url_blocker_start
					;;
				stop)
					url_blocker_stop
					;;
				restart)
					url_blocker_stop
					url_blocker_start
					;;
				*)
					;;
				esac

				# unlock the file lock
				flock -u 8
				break
			else
				# wait for 100 ms
				retry_count=$((retry_count+1))
				usleep 100000
			fi
		done

		if [ ${retry_count} -gt ${retry_max} ]; then
			/usr/bin/logger -p err "Faild to lock [${szf_script_lock}] for action [${action}]"
		fi
	)8> ${szf_script_lock}
}

# make sure that the parameters are correctly specified
case "$1" in
	start|stop|restart)
		action_main $1
		;;
	*)
		echo "Usage: $SCRIPT_NAME [start|stop|restart]"
		RETVAL=1
		;;
esac

exit $RETVAL
